using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LifePreserverController : MonoBehaviour {
  private LifePreserverManager lifePreserverManager;
  private StateTracker stateTracker;
  private Walker walker;
  private Collider2D _collider;

  void Awake() {
    lifePreserverManager = LifePreserverManager.Instance;
    stateTracker = GetComponent<StateTracker>();
    walker = GetComponent<Walker>();
  }

  void OnEnable() {
    _collider = GetComponent<Collider2D>();
    _collider.enabled = false;
    // Life preservers always float when instantiated.
    stateTracker.MoveState("falling");
  }

  void Update() {
    if (!walker.autoWalk && stateTracker.state == "ready") {
      walker.autoWalk = true;
      _collider.enabled = true;
    }
    if (walker.autoWalk && stateTracker.state != "ready") {
      walker.autoWalk = false;
      _collider.enabled = false;
    }
  }

  public bool IsMounted() {
    return stateTracker.state == "mounted";
  }

  public void SetMounted() {
    stateTracker.MoveState("mounted");
    _collider.enabled = false;
  }

  public void DestroySelf() {
    lifePreserverManager.DestroyPreserver(gameObject);
  }
}
