using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LifePreserverManager : MonoBehaviour {
  public static LifePreserverManager Instance;

  public GameObject lifePreserverPrefab;

  private GameObjectPool preserverPool;

  const int MOUSE_BUTTON = 0;

  void Awake() {
    if (LifePreserverManager.Instance) DestroyImmediate(this);
    LifePreserverManager.Instance = this;
    preserverPool = new GameObjectPool(lifePreserverPrefab);
  }

  void Update() {
    CheckForClick();
  }

  void CheckForClick() {
    if (Input.GetMouseButtonDown(MOUSE_BUTTON)) {
      var surfacePoint = GetSurfacePoint(Input.mousePosition);
      var lifePreserver = CreatePreserver(surfacePoint + Vector3.back * 100f);
      LeanTween.move(lifePreserver, surfacePoint, 1f).setOnComplete(() => {
        lifePreserver.GetComponent<StateTracker>().MoveState("ready");
      });
    }
  }

  Vector3 GetSurfacePoint(Vector3 mousePosition) {
    var mouseAtCamDist = new Vector3(mousePosition.x, mousePosition.y, -Camera.main.transform.position.z);
    var mouseWorldPoint = Camera.main.ScreenToWorldPoint(mouseAtCamDist);
    var surfacePoint = new Vector3(mouseWorldPoint.x, mouseWorldPoint.y, 0f);
    return surfacePoint;
    // var hit = Physics2D.Raycast(mouseWorldPoint, Vector2.zero, LayerMask.NameToLayer("OceanSurface"));
    // return new Vector3(hit.point.x, hit.point.y, 0f);
  }

  GameObject CreatePreserver(Vector3 position) {
    var lifePreserver = preserverPool.Instantiate(lifePreserverPrefab);
    lifePreserver.transform.position = position;
    lifePreserver.transform.parent = transform;
    return lifePreserver;
  }

  public void DestroyPreserver(GameObject preserver) {
    preserver.transform.parent = transform;
    preserverPool.Destroy(preserver);
  }
}
