using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Mounter : MonoBehaviour {
  public List<MountPair> mountMap;
  public List<GameObject> mountedItems;

  private Dictionary<string, Transform> _mountMap;

  void Awake() {
    _mountMap = new Dictionary<string, Transform>();
    foreach (var pair in mountMap) {
      _mountMap.Add(pair.tagName, pair.mountPoint);
    }
  }

  // void Update() {

  // }

  public void Mount(GameObject obj) {
    var mountPoint = GetMountPointForTag(obj.tag);
    if (mountPoint != null) {
      var originalPos = obj.transform.position;
      obj.transform.parent = mountPoint;
      LeanTween.move(obj, mountPoint, 0.75f).setEaseOutBack();
      obj.SendMessage("SetMounted");
      mountedItems.Add(obj);
    }
  }

  public void DestroyAll() {
    mountedItems.ForEach(item => item.SendMessage("DestroySelf", SendMessageOptions.DontRequireReceiver));
  }

  Transform GetMountPointForTag(string tag) {
    Transform mountPoint;
    if (_mountMap.TryGetValue(tag, out mountPoint)) {
      return mountPoint;
    } else {
      return null;
    }
  }

  [System.Serializable]
  public class MountPair {
    public string tagName;
    public Transform mountPoint;

    public MountPair(string tag, Transform mount) {
      tagName = tag;
      mountPoint = mount;
    }
  }
}

