using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SwimmerController : MonoBehaviour
{
    public float struggleTimer;
    public float drownTimer;
    public float shakeOffset;
    [Range(0,0.99f)]
    public float shakeSpeed;

    private SwimmerManager swimmerManager;
    private Mounter mounter;
    private StateTracker stateTracker;
    private Walker walker;
    private SpriteRenderer spriteRenderer;

    void Awake() {
      swimmerManager = SwimmerManager.Instance;
      mounter = GetComponent<Mounter>();
      stateTracker = GetComponent<StateTracker>();
      walker = GetComponent<Walker>();
      spriteRenderer = GetComponentInChildren<SpriteRenderer>();
    }

    void OnEnable() {
      stateTracker.state = "swimming";
      transform.localScale = Vector3.one;
      FadeIn();
    }

    void Update() {
      switch (stateTracker.state) {
        case "swimming":
          // StruggleCheck
          struggleTimer -= Time.deltaTime;
          if (struggleTimer <= 0f) {
            // TODO: Start struggle animation
            stateTracker.MoveState("struggling");
            Vector3 shakeVector = Random.insideUnitCircle * shakeOffset;
            LeanTween.move(gameObject, transform.position + shakeVector, 1f - shakeSpeed)
              .setEase(LeanTween.shake)
              .setLoopClamp()
              .setRepeat(-1);
            LeanTween.alpha(gameObject, 0.25f, 0.5f)
              .setLoopClamp()
              .setLoopPingPong(-1);
              // .setRepeat(-1);
          }
          break;

        case "struggling":
          // DrownCheck
          drownTimer -= Time.deltaTime;
          if (drownTimer <= 0f) {
            stateTracker.MoveState("drowning");
            LeanTween.cancel(gameObject);
            LeanTween.scale(gameObject, Vector3.one * 0.05f, 1f).setEaseInBack();
            LeanTween.alpha(gameObject, 0f, 1f)
              .setEaseInQuart()
              .setOnComplete(() => stateTracker.MoveState("lost"));
          }
          break;

        case "saved":
          // SavedCheck
          if (!spriteRenderer.isVisible) {
            // Yay! We're saved!
            /*
              - Animate the save!
              - Signal to SwimmerManager that this one's saved
              - Recycle life preserver
            */
            mounter.DestroyAll();
            swimmerManager.SwimmerSaved(gameObject);
          }
          break;

        case "drowning":
          // Wait for drown
          break;

        case "lost":
          mounter.DestroyAll();
          swimmerManager.SwimmerLost(gameObject);
          break;
      }



    }

    void OnTriggerEnter2D(Collider2D c) {
      if (IsStruggling() && c.gameObject.tag == "LifePreserver") {
        stateTracker.MoveState("saved");
        LeanTween.cancel(gameObject);
        LeanTween.alpha(gameObject, 1f, 0.2f);
        mounter.Mount(c.gameObject);
        walker.SetTarget(FindRescueTarget());
      }
    }

    Vector3 FindRescueTarget() {
      return transform.position * 3f;
    }

    public bool IsSwimming() {
      return stateTracker.state == "swimming";
    }

    public bool IsStruggling() {
      return stateTracker.state == "struggling";
    }

    public bool IsSaved() {
      return stateTracker.state == "saved";
    }

    public bool IsDrowning() {
      return stateTracker.state == "drowning";
    }

    void FadeIn() {
      LeanTween.alpha(gameObject, 0f, 0f)
        .setOnComplete(() => LeanTween.alpha(gameObject, 1f, 0.5f));
    }
}
