using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Walker : MonoBehaviour {
    public string avoidSpriteTag;
    public float thinkDelaySecs;
    public float walkSpeed;
    public float maxRandomWalkAngle;
    public float collisionFastMultiplier;
    public float collisionSlowMultiplier;
    public float collisionAvoidAnglePct;
    public float beelineRandomWalkAngle;
    public float beelineSpeedFactor;
    public AnimationCurve walkSpeedCurve = AnimationCurve.Constant(0f, 1f, 1f);
    public AnimationCurve turnAngleCurve = AnimationCurve.Constant(0f, 1f, 1f);
    public bool autoWalk;

    private Vector3 walkVector;
    private Vector3 walkTarget;
    private float walkSpeedFactor;
    private IEnumerator thinkerCoroutine;
    private List<Transform> avoidList;

    void OnEnable() {
      avoidList = new List<Transform>();
      walkVector = randomVector();
      walkSpeedFactor = Random.Range(0.9f, 1.1f);
      walkTarget = Vector3.zero;
      thinkerCoroutine = Thinker();
      StartCoroutine(thinkerCoroutine);
      Think();
    }

    void OnDisable() {
      StopCoroutine(thinkerCoroutine);
      walkVector = Vector3.zero;
    }

    void Update() {
      if (autoWalk) {
        Walk(Time.deltaTime);
      }

      if (walkTarget != Vector3.zero) {
        Debug.DrawLine(transform.position, walkTarget, Color.yellow);
      }
    }

    void OnTriggerEnter2D(Collider2D c) {
      if (c.gameObject.tag != avoidSpriteTag) return;
      avoidList.Add(c.gameObject.transform);
    }

    void OnTriggerExit2D(Collider2D c) {
      if (c.gameObject.tag != avoidSpriteTag) return;
      avoidList.Remove(c.gameObject.transform);
    }

    public void SetTarget(Vector3 target) {
      walkTarget = target;
      avoidList.Clear();
    }

    IEnumerator Thinker() {
      // var delay = new WaitForSeconds(thinkDelaySecs);
      while(true) {
        yield return new WaitForSeconds(thinkDelaySecs);
        Think();
      }
    }

    void Think() {
      float maxAngle;
      float maxSpeed;

      if (walkTarget == Vector3.zero) {
        maxAngle = maxRandomWalkAngle * turnAngleCurve.Evaluate(Time.time);
        maxSpeed = walkSpeed * walkSpeedCurve.Evaluate(Time.time * walkSpeedFactor);
      } else {
        maxAngle = beelineRandomWalkAngle * turnAngleCurve.Evaluate(Time.time);
        maxSpeed = walkSpeed * beelineSpeedFactor * walkSpeedCurve.Evaluate(Time.time * walkSpeedFactor);
      }

      Think(maxAngle, maxSpeed/2f, maxSpeed);
    }

    void Think(float angle, float minMagnitude, float maxMagnitude) {
      float rotateByAngle = Random.Range(-angle/2f, angle/2f);
      float newMagnitude = Random.Range(minMagnitude, maxMagnitude);
      // If we have a target, point in that direction and then randomly rotate.
      // Otherwise, start with our current heading and then rotate.
      var direction = walkTarget == Vector3.zero ? walkVector : walkTarget - transform.position;
      walkVector = walkVector.normalized * newMagnitude;
      walkVector = Quaternion.Euler(0, 0, rotateByAngle) * walkVector;
    }

    void Walk(float multiplier) {
      var slowMult = multiplier * collisionSlowMultiplier;
      var fastMult = multiplier * collisionFastMultiplier;
      foreach (var avoidObj in avoidList) {
        var dirAvoid = (avoidObj.position - transform.position).normalized;
        if (Vector3.Dot(dirAvoid, walkVector.normalized) > collisionAvoidAnglePct) {
          multiplier = slowMult;
        }
      }
      if (avoidList.Count > 0 && multiplier != slowMult) {
        multiplier = fastMult;
      }
      gameObject.transform.Translate(walkVector * multiplier);
    }

    private Vector3 randomVector() {
      return new Vector3(Random.Range(0f, 1f), Random.Range(0f, 1f), 0).normalized;
    }
}
